package eu.jovian_hersemeule.dev.squad.squad_composer.controllers;

import org.springframework.web.bind.annotation.RestController;
import eu.jovian_hersemeule.dev.squad.squad_composer.data.projections.PlayerItem;
import eu.jovian_hersemeule.dev.squad.squad_composer.data.projections.SquadItem;
import eu.jovian_hersemeule.dev.squad.squad_composer.data.repos.PlayerRepository;

import java.util.List;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

@RestController
@RequestMapping("/players")
public class PlayerController {

    final private PlayerRepository repo;

    PlayerController(final PlayerRepository playerRepository) {
        this.repo = playerRepository;
    }

    @GetMapping("/")
    public List<PlayerItem> listPlayers() {
        return repo.findAll().stream()
                .map(PlayerItem::project)
                .toList();
    }

    @GetMapping("/{id}")
    public PlayerItem getASpecificPlayer(@PathVariable final long id) {
        return repo.findById(id)
                .map(PlayerItem::project)
                .orElseThrow(() -> new RuntimeException("Player not found"));
    }

    @GetMapping("/{id}/squads")
    public List<SquadItem> getSquadsOfPlayer(@PathVariable final long id) {
        return repo.findById(id).stream()
                .flatMap(player -> player.getSquads().stream())
                .map(SquadItem::project)
                .toList();
    }
    
}
